@@ -5,17 +5,17 @@ from json import dump, load
 from logging import DEBUG, INFO, basicConfig, getLogger
 from os import getenv, name
 from pathlib import Path
-from socket import timeout
-from typing import Iterable, NamedTuple
+from socket import gaierror, timeout
+from typing import Iterable, NamedTuple, Type
 
 from gi import require_version
 require_version('Gtk', '3.0')
-# pylint: disable=C0413
 from gi.repository import Gtk
 
-from rcon.client import Client
+from rcon import battleye, source
+from rcon.client import BaseClient
 from rcon.config import LOG_FORMAT
-from rcon.exceptions import RequestIdMismatch, WrongPassword
+from rcon.exceptions import SessionTimeout, WrongPassword
 
 
 __all__ = ['main']
@@ -37,10 +37,18 @@ def get_args() -> Namespace:
     """Parses the command line arguments."""
 
     parser = ArgumentParser(description='A minimalistic, GTK-based RCON GUI.')
-    parser.add_argument('-d', '--debug', action='store_true',
-                        help='print additional debug information')
-    parser.add_argument('-t', '--timeout', type=float, metavar='seconds',
-                        help='connection timeout in seconds')
+    parser.add_argument(
+        '-B', '--battleye', action='store_true',
+        help='use BattlEye RCon instead of Source RCON'
+    )
+    parser.add_argument(
+        '-d', '--debug', action='store_true',
+        help='print additional debug information'
+    )
+    parser.add_argument(
+        '-t', '--timeout', type=float, metavar='seconds',
+        help='connection timeout in seconds'
+    )
     return parser.parse_args()
 
 
@@ -70,7 +78,7 @@ class GUI(Gtk.Window):  # pylint: disable=R0902
         self.host.set_placeholder_text('Host')
         self.grid.attach(self.host, 0, 0, 1, 1)
 
-        self.port = Gtk.Entry()
+        self.port = Gtk.SpinButton.new_with_range(0, 65535, 1)
         self.port.set_placeholder_text('Port')
         self.grid.attach(self.port, 1, 0, 1, 1)
 
@@ -89,6 +97,7 @@ class GUI(Gtk.Window):  # pylint: disable=R0902
 
         self.result = Gtk.TextView()
         self.result.set_wrap_mode(Gtk.WrapMode.WORD)
+        self.result.set_property('editable', False)
         self.grid.attach(self.result, 0, 2, 2, 1)
 
         self.savepw = Gtk.CheckButton(label='Save password')
@@ -96,6 +105,11 @@ class GUI(Gtk.Window):  # pylint: disable=R0902
 
         self.load_gui_settings()
 
+    @property
+    def client_cls(self) -> Type[BaseClient]:
+        """Returns the client class."""
+        return battleye.Client if self.args.battleye else source.Client
+
     @property
     def result_text(self) -> str:
         """Returns the result text."""
@@ -117,7 +131,7 @@ class GUI(Gtk.Window):  # pylint: disable=R0902
         """Returns the GUI settings as a dict."""
         json = {
             'host': self.host.get_text(),
-            'port': self.port.get_text(),
+            'port': self.port.get_value_as_int(),
             'command': self.command.get_text(),
             'result': self.result_text,
             'savepw': (savepw := self.savepw.get_active())
@@ -132,16 +146,16 @@ class GUI(Gtk.Window):  # pylint: disable=R0902
     def gui_settings(self, json: dict):
         """Sets the GUI settings."""
         self.host.set_text(json.get('host', ''))
-        self.port.set_text(json.get('port', ''))
+        self.port.set_value(json.get('port', 0))
         self.passwd.set_text(json.get('passwd', ''))
         self.command.set_text(json.get('command', ''))
         self.result_text = json.get('result', '')
         self.savepw.set_active(json.get('savepw', False))
 
-    def load_gui_settings(self) -> dict:
+    def load_gui_settings(self) -> None:
         """Loads the GUI settings from the cache file."""
         try:
-            with CACHE_FILE.open('r') as cache:
+            with CACHE_FILE.open('rb') as cache:
                 self.gui_settings = load(cache)
         except FileNotFoundError:
             LOGGER.warning('Cache file not found: %s', CACHE_FILE)
@@ -153,7 +167,7 @@ class GUI(Gtk.Window):  # pylint: disable=R0902
     def save_gui_settings(self):
         """Saves the GUI settings to the cache file."""
         try:
-            with CACHE_FILE.open('w') as cache:
+            with CACHE_FILE.open('wb') as cache:
                 dump(self.gui_settings, cache)
         except PermissionError:
             LOGGER.error('Insufficient permissions to read: %s', CACHE_FILE)
@@ -164,39 +178,20 @@ class GUI(Gtk.Window):  # pylint: disable=R0902
             transient_for=self,
             message_type=Gtk.MessageType.ERROR,
             buttons=Gtk.ButtonsType.OK,
-            text=message)
+            text=message
+        )
         message_dialog.run()
         message_dialog.destroy()
 
-    def get_rcon_params(self) -> RCONParams:
-        """Returns all settings as a dict."""
-        if not (host := self.host.get_text().strip()):
-            raise ValueError('No host specified.')
-
-        if not (port := self.port.get_text().strip()):
-            raise ValueError('No port specified.')
-
-        try:
-            port = int(port)
-        except ValueError:
-            raise ValueError('Invalid port specified.') from None
-
-        if not 0 <= port <= 65535:
-            raise ValueError('Invalid port specified.')
-
-        if not (command := self.command.get_text().strip()):
-            raise ValueError('No command entered.')
-
-        command = tuple(filter(None, command.split()))
-        return RCONParams(host, port, self.passwd.get_text(), command)
-
     def run_rcon(self) -> str:
         """Returns the current RCON settings."""
-        params = self.get_rcon_params()
-
-        with Client(params.host, params.port, timeout=self.args.timeout,
-                    passwd=params.passwd) as client:
-            return client.run(*params.command)
+        with self.client_cls(
+                self.host.get_text().strip(),
+                self.port.get_value_as_int(),
+                timeout=self.args.timeout,
+                passwd=self.passwd.get_text()
+        ) as client:
+            return client.run(*self.command.get_text().strip().split())
 
     def on_button_clicked(self, _):
         """Runs the client."""
@@ -204,16 +199,16 @@ class GUI(Gtk.Window):  # pylint: disable=R0902
             result = self.run_rcon()
         except ValueError as error:
             self.show_error(str(error))
+        except gaierror as error:
+            self.show_error(error.strerror)
         except ConnectionRefusedError:
             self.show_error('Connection refused.')
         except (TimeoutError, timeout):
             self.show_error('Connection timed out.')
         except WrongPassword:
             self.show_error('Wrong password.')
-        except RequestIdMismatch as mismatch:
-            self.show_error(
-                'Request ID mismatch.\n'
-                f'Expected {mismatch.sent}, but got {mismatch.received}.')
+        except SessionTimeout:
+            self.show_error('Session timed out.')
         else:
             self.result_text = result
 
